﻿// Copyright (C) 2008 - 2011 CASIO Europe GmbH
//
// This file is part of SoftkeysWM.
//
// SoftkeysWM is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// System Requirements: Windows Embedded Compact 6.5 (IT-300)
// See also http://msdn.microsoft.com/en-us/library/bb431750.aspx
//
// Version 1.1.0.0
///////////////////////////////////////////////////////////////////

using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Calib;

namespace SoftkeysWM
{
    /// <summary>
    /// This samples shows how to assign new key codes.
    /// </summary>
    public partial class MainForm : Form
    {
        private const string HelpText =
            "Press F1 to F4 to perform the desired actions.\n\n" +
            "Key F1: VK_LWIN (Home screen)\n" +
            "Key F2: VK_F1 (Softkey 1)\n" +
            "Key F3: VK_F2 (Softkey 2)\n" +
            "Key F4: VK_RWIN + VK_F6 (OK/Close)\n\n" +
            "Enable/Disable the Software Input Panel (SIP) by pressing Fn + 0.";

        public MainForm()
        {
            InitializeComponent();

            this.lblInstruction.Text = HelpText;
            ConfigureKeys();
        }

        private void MainForm_Closing(object sender, CancelEventArgs e)
        {
            int result;

            ConfigureKeys(false);

            // Disable any user settings.
            result = SystemLibNet.Api.SysSetUserDefineKeyState(false);
        }

        private void ConfigureKeys()
        {
            ConfigureKeys(true);
        }

        private void ConfigureKeys(bool init)
        {
            int result;

            // If the form has a main menu with two buttons (softkeys),
            // the left softkey is assigned to key code F1, and
            // the right softkey is assigned to key code F2.
            // However, actual hardware layout does not fit softkey bar.
            // Hence, we must make new assignment:
            // Hardware key [F1]: VK_LWIN -> Home screen.
            // Hardware key [F2]: VK_F1 -> Softkey 1.
            // Hardware key [F3]: VK_F2 -> Softkey 2.
            // Hardware key [F4]: VK_RWIN + VK_F6 -> <OK/Close> hotkey code.
            if (init == true)
            {
                int[] keyBufferF1 = new int[16];
                int[] keyBufferF2 = new int[16];
                int[] keyBufferF3 = new int[16];
                int[] keyBufferF4 = new int[16]; 

                keyBufferF1[0] = (int)Keys.LWin;

                result = SystemLibNet.Api.SysGetDefaultKey(
                    SystemLibNet.Def.KEYID_F1,
                    keyBufferF2);

                result = SystemLibNet.Api.SysGetDefaultKey(
                    SystemLibNet.Def.KEYID_F2,
                    keyBufferF3);

                // VK_F5 is free, because there is no additional mapping defined for Windows Mobile.
                keyBufferF4[0] = (int)Keys.F5;  

                for (short i = SystemLibNet.Def.KEY_MODE_NUM; i <= SystemLibNet.Def.KEY_MODE_ALPHAS; i++)
                {
                    result = SystemLibNet.Api.SysSetUserDefineKey(
                        i,
                        SystemLibNet.Def.KEYID_F1,
                        keyBufferF1);

                    result = SystemLibNet.Api.SysSetUserDefineKey(
                        i,
                        SystemLibNet.Def.KEYID_F2,
                        keyBufferF2);

                    result = SystemLibNet.Api.SysSetUserDefineKey(
                        i,
                        SystemLibNet.Def.KEYID_F3,
                        keyBufferF3);

                    result = SystemLibNet.Api.SysSetUserDefineKey(
                        i,
                        SystemLibNet.Def.KEYID_F4,
                        keyBufferF4);
                }

                // New assignments do not appliy to Fn-Key mode, when
                // a specific keys is pressed immediately after the Fn-Key.
                // If you do not want that, disable the Fn-Key mode here.
                //SystemLibNet.Api.SysSetFnKeyLock(true);
            }
            else
            {
                for (short i = SystemLibNet.Def.KEY_MODE_NUM; i <= SystemLibNet.Def.KEY_MODE_ALPHAS; i++)
                {
                    result = SystemLibNet.Api.SysDeleteUserDefineKeyEx(
                        i,
                        SystemLibNet.Def.KEYID_F1);

                    result = SystemLibNet.Api.SysDeleteUserDefineKeyEx(
                        i,
                        SystemLibNet.Def.KEYID_F2);

                    result = SystemLibNet.Api.SysDeleteUserDefineKeyEx(
                        i,
                        SystemLibNet.Def.KEYID_F3);

                    result = SystemLibNet.Api.SysDeleteUserDefineKeyEx(
                        i,
                        SystemLibNet.Def.KEYID_F4);
                }

                //SystemLibNet.Api.SysSetFnKeyLock(false);
            }

            // Inform the keyboard driver about changes.
            result = SystemLibNet.Api.SysSetUserDefineKeyState(true);
        }

        private void menuLeft_Click(object sender, EventArgs e)
        {
            this.statusBar.Text = "Left softkey pressed.";
        }

        private void menuRight_Click(object sender, EventArgs e)
        {
            this.statusBar.Text = "Right softkey pressed.";
        }

        private void MainForm_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.F5:
                    KeyboardEmulation.VkDone();
                    break;
                default:
                    break;
            }
        }
    }
}